@@ -68,6 +68,7 @@ PPListBox::PPListBox(pp_int32 id, PPScreen* parentScreen, EventListenerInterface
 	backGroundButtonColor(&PPUIConfig::getInstance()->getColor(PPUIConfig::ColorListBoxBackground)),
 	// default textcolor 
 	textColor(&PPUIConfig::getInstance()->getColor(PPUIConfig::ColorStaticText)),
+	indexColor(&PPUIConfig::getInstance()->getColor(PPUIConfig::ColorStaticText)),
 
 	editable(editable),
 	scrollable(scrollable),
@@ -75,8 +76,12 @@ PPListBox::PPListBox(pp_int32 id, PPScreen* parentScreen, EventListenerInterface
 	autoHideHScroll(true),
 
 	showIndex(false),
+	hexIndex(true),
 	indexBaseCount(1),
 
+	highLightRangeStart(-1),
+	highLightRangeStop(-1),
+
 	showSelectionAlways(showSelectionAlways),
 	selectionVisible(true),
 	onlyShowIndexSelection(false),
@@ -237,14 +242,22 @@ void PPListBox::paint(PPGraphicsAbstract* g)
 
 			g->setRect(currentRect);
 		}
-
 		if (showIndex)
 		{
-			char hexIndex[10];
-		
-			PPTools::convertToHex(hexIndex, i + indexBaseCount, maxDigits);
-			
-			g->drawString(hexIndex, location.x + 2, pos);
+			char strIndex[10];
+
+			if( hexIndex ){
+				PPTools::convertToHex(strIndex, i + indexBaseCount, maxDigits);
+			}else{
+				PPTools::convertToDec(strIndex, i + indexBaseCount, maxDigits);
+			}
+			g->setColor(*indexColor);	
+
+			if( i >= highLightRangeStart && i <= highLightRangeStop ){ 
+				g->setColor(PPUIConfig::getInstance()->getColor(PPUIConfig::ColorDefaultButtonText));
+			}
+
+			g->drawString(strIndex, location.x + 2, pos);
 		}
 		
 		PPRect currentRect = g->getRect();
@@ -275,6 +288,10 @@ void PPListBox::paint(PPGraphicsAbstract* g)
 
 		g->setColor(colorQueryListener ? colorQueryListener->getColor(i, *this) : *textColor);
 
+		if( i >= highLightRangeStart && i <= highLightRangeStop ){ 
+			g->setColor(PPUIConfig::getInstance()->getColor(PPUIConfig::ColorDefaultButtonText));
+		}
+
 		g->drawString(*items->get(i), location.x + xOffset - (startPos*font->getCharWidth()), pos);
 
 		g->setRect(currentRect);
@@ -774,10 +791,11 @@ placeCursor:
 
 
 		case eKeyDown:
-		{	
-			if (caughtControl)
+		{
+			// prevent double triggers for kb shortcuts directly adressing this control
+			if (caughtControl )
 				break;
-		
+
 			if (selectionIndex < 0 ||
 				selectionIndex >= items->size()/* ||
 				columnSelectionStart < 0*/)
@@ -1026,8 +1044,9 @@ pp_int32 PPListBox::handleEvent(PPObject* sender, PPEvent* event)
 			pp_int32 visibleItems = getNumVisibleItems();
 
 			startIndex += event->getMetaData();
-			if (startIndex + visibleItems > items->size())
+			if (startIndex + visibleItems >= items->size())
 				startIndex = items->size() - visibleItems;
+      else startIndex++;
 
 			float v = (float)(items->size() - visibleItems);
 
@@ -1062,6 +1081,7 @@ pp_int32 PPListBox::handleEvent(PPObject* sender, PPEvent* event)
 			startIndex -= event->getMetaData();
 			if(startIndex < 0)
 				startIndex = 0;
+      else startIndex--;
 			
 			pp_int32 visibleItems = getNumVisibleItems();
 			
