@@ -39,6 +39,10 @@ class DialogHelp : public PPDialogBase
 {
 private:
 	class PPListBox* listBox;
+	pp_int32 lineMilkySynth;
+	pp_int32 lineShortcuts;
+	pp_int32 linePatternFX;
+	pp_int32 handleEvent(PPObject* sender, PPEvent* event);
 
 public:
 	DialogHelp(PPScreen* screen, 
@@ -46,8 +50,11 @@ public:
 				  pp_int32 id,
 				  const PPString& caption,
 				  bool okCancel = false);
+	~DialogHelp();
 	
 	PPListBox* getListBox() { return listBox; }
+	void show(bool show);
+	static pp_int32 position;
 
 };
 
