@@ -149,6 +149,7 @@ ModuleEditor::ModuleEditor() :
 	
 	// create sample editor
 	sampleEditor = new SampleEditor();
+	//printf("samplerate=%i\n",PlayerMaster::getPreferredSampleRate());
 	sampleEditor->addNotificationListener(changesListener);
 	reloadSample(0, 0);
 	
@@ -487,15 +488,16 @@ void ModuleEditor::convertInstrument(mp_sint32 i)
 		//-----------------------------------------------------------
 		// this will only work if there are 16 sample per instrument
 		//-----------------------------------------------------------
+		pp_int32 defaultVolfade = 512; // default fade (better UX when enabling envelope or synth)
 		for (j = 0; j < 16; j++)
 		{
 			module->smp[i*16+j].venvnum = venvIndex + 1;
 			module->smp[i*16+j].penvnum = penvIndex + 1;
 			// default fade out to cut
-			module->smp[i*16+j].volfade = 0xFFFF;
+			module->smp[i*16+j].volfade = defaultVolfade; 
 		}
 
-		instruments[i].volfade = 0xFFFF >> 1;
+		instruments[i].volfade = defaultVolfade; 
 		instruments[i].vibtype = 0;
 		instruments[i].vibrate = 0;
 		instruments[i].vibdepth = 0;
@@ -1562,8 +1564,14 @@ bool ModuleEditor::loadInstrument(const SYSCHAR* fileName, mp_sint32 index)
 {
 	ASSERT(index < module->header.insnum);
 
+	PPString file = PPString(fileName);
+
+	if( file.compareExtensions( PPString(".sf2") ) == 0 ){
+		// for now we reroute to loadSample
+		loadSample(fileName, index, 0, 1 );
+	}
+
 	XIInstrument* ins = new XIInstrument();
-	
 	bool res = ins->load(fileName) == 0; 
 	
 	if (res)
@@ -1576,7 +1584,6 @@ bool ModuleEditor::loadInstrument(const SYSCHAR* fileName, mp_sint32 index)
 	}
 	
 	delete ins;
-
 	return res;
 }
 
@@ -2482,9 +2489,9 @@ void ModuleEditor::adjustSampleOffsetCommandAfterSampleSizeChange(TXMSample *sam
 	mp_ubyte* lastIns = new mp_ubyte[module->header.channum];
 	memset(lastIns, 0, module->header.channum);
 
-	for (mp_sint32 l = 0; l < module->header.ordnum; l++)
+	for (mp_sint32 i = 0; i < MAX_PATTERNS; i++)
 	{
-		TXMPattern* pattern = &module->phead[module->header.ord[l]];
+		TXMPattern* pattern = &module->phead[i];
 
 		if (pattern->patternData == NULL)
 			continue;
