@@ -37,7 +37,7 @@ void PatternEditorControl::initKeyBindings()
 	eventKeyDownBindingsMilkyTracker = new PPKeyBindings<TPatternEditorKeyBindingHandler>;
 
 	// Key-down bindings MilkyTracker
-	eventKeyDownBindingsMilkyTracker->addBinding(VK_LEFT, 0, &PatternEditorControl::eventKeyDownBinding_LEFT);
+	eventKeyDownBindingsMilkyTracker->addBinding(VK_LEFT,  0, &PatternEditorControl::eventKeyDownBinding_LEFT);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_RIGHT, 0, &PatternEditorControl::eventKeyDownBinding_RIGHT);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_UP, 0xFFFF, &PatternEditorControl::eventKeyDownBinding_UP);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_DOWN, 0xFFFF, &PatternEditorControl::eventKeyDownBinding_DOWN);
@@ -55,6 +55,13 @@ void PatternEditorControl::initKeyBindings()
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_LEFT, KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_LEFT);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_RIGHT, KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_RIGHT);
 
+	// compensate for lack of FT2 channel/column selectors:
+	//  1. expand select downward when adding shift while pressing ctrl+arrowkeys
+	eventKeyDownBindingsMilkyTracker->addBinding(VK_LEFT,  KeyModifierCTRL | KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_SelectColumn);
+	eventKeyDownBindingsMilkyTracker->addBinding(VK_RIGHT,  KeyModifierCTRL | KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_SelectColumn);
+	eventKeyDownBindingsMilkyTracker->addBinding(VK_LEFT,  KeyModifierCTRL|KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_LEFT);
+	eventKeyDownBindingsMilkyTracker->addBinding(VK_RIGHT, KeyModifierCTRL|KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_RIGHT);
+
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_DELETE, KeyModifierSHIFT, &PatternEditorControl::eventKeyDownBinding_DeleteNoteVolumeAndEffect);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_DELETE, KeyModifierCTRL, &PatternEditorControl::eventKeyDownBinding_DeleteVolumeAndEffect);
 	eventKeyDownBindingsMilkyTracker->addBinding(VK_DELETE, KeyModifierALT, &PatternEditorControl::eventKeyDownBinding_DeleteEffect);
@@ -72,11 +79,14 @@ void PatternEditorControl::initKeyBindings()
 	eventKeyDownBindingsMilkyTracker->addBinding('C', KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_Copy);
 	eventKeyDownBindingsMilkyTracker->addBinding('V', KeyModifierSHIFT|KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_TransparentPaste);
 	eventKeyDownBindingsMilkyTracker->addBinding('V', KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_Paste);
+	eventKeyDownBindingsMilkyTracker->addBinding('V', KeyModifierCTRL|KeyModifierALT, &PatternEditorControl::eventKeyCharBinding_PasteStepFill);
 	eventKeyDownBindingsMilkyTracker->addBinding('A', KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_SelectAll);
 	eventKeyDownBindingsMilkyTracker->addBinding('M', KeyModifierSHIFT, &PatternEditorControl::eventKeyCharBinding_MuteChannel);
 	eventKeyDownBindingsMilkyTracker->addBinding('M', KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_MuteChannel);
 	eventKeyDownBindingsMilkyTracker->addBinding('M', KeyModifierSHIFT|KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_InvertMuting);
 	eventKeyDownBindingsMilkyTracker->addBinding('I', KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_Interpolate);
+	eventKeyDownBindingsMilkyTracker->addBinding('R', KeyModifierCTRL, &PatternEditorControl::eventKeyCharBinding_PasteStepFill);
+
 
 	// Scancode bindings
 	scanCodeBindingsMilkyTracker = new PPKeyBindings<TPatternEditorKeyBindingHandler>;
@@ -673,7 +683,11 @@ void PatternEditorControl::handleKeyDown(pp_uint16 keyCode, pp_uint16 scanCode,
 		if (number == -1 && ::getKeyModifier() == 0)
 			number = ScanCodeToNote(scanCode);
 
+		int ins = patternEditor->getCurrentActiveInstrument();
+		// apply RR on instrument if any
+		patternEditor->setCurrentInstrument( rr.ins_size == 0 ? ins : rr.ins_start + rr.ins_index );
 		patternEditor->writeNote(number, true, this);
+		patternEditor->setCurrentInstrument( ins ); // undo RR 
 	}
 	else
 		handleDeleteKey(keyCode, number);
@@ -683,12 +697,12 @@ void PatternEditorControl::handleKeyDown(pp_uint16 keyCode, pp_uint16 scanCode,
 
 void PatternEditorControl::handleKeyChar(pp_uint8 character)
 {
-	PatternEditorTools::Position& cursor = patternEditor->getCursor();
-	pp_int32 number = -1;
 
 	// prevent unnecessary screen refreshing through listener callback
 	// remember to reset this when leaving this function
 	patternEditor->setLazyUpdateNotifications(true);
+	PatternEditorTools::Position& cursor = patternEditor->getCursor();
+	pp_int32 number = -1;
 
 	if ((cursor.inner == 1 || cursor.inner == 2))
 	{
@@ -806,8 +820,10 @@ void PatternEditorControl::handleKeyChar(pp_uint8 character)
 	}
 
 	// If the input had an effect, ensure the PatternEditorControl is repainted
-	if (number != -1)
+	if (number != -1){
+		updateStatus();
 		assureUpdate = true;
+	}
 
 cleanUp:
 	patternEditor->setLazyUpdateNotifications(false);
@@ -859,6 +875,7 @@ void PatternEditorControl::eventKeyDownBinding_LEFT()
 			}
 		}
 	}
+	updateStatus();
 }
 
 void PatternEditorControl::eventKeyDownBinding_RIGHT()
@@ -888,6 +905,7 @@ void PatternEditorControl::eventKeyDownBinding_RIGHT()
 			}
 		}
 	}
+	updateStatus();
 }
 
 void PatternEditorControl::eventKeyDownBinding_UP()
@@ -915,8 +933,8 @@ void PatternEditorControl::eventKeyDownBinding_UP()
 		cursor.row = pattern->rows-1;
 	}
 
-	if (!res)
-		notifyUpdate(AdvanceCodeSelectNewRow);
+	if (!res) notifyUpdate(AdvanceCodeSelectNewRow);
+	updateStatus();
 }
 
 void PatternEditorControl::eventKeyDownBinding_DOWN()
@@ -943,8 +961,8 @@ void PatternEditorControl::eventKeyDownBinding_DOWN()
 		cursor.row = 0;
 	}
 
-	if (!res)
-		notifyUpdate(AdvanceCodeSelectNewRow);
+	if (!res) notifyUpdate(AdvanceCodeSelectNewRow);
+	updateStatus();
 }
 
 void PatternEditorControl::eventKeyDownBinding_PRIOR()
@@ -971,8 +989,8 @@ void PatternEditorControl::eventKeyDownBinding_PRIOR()
 		cursor.row = /*wrapAround ? pattern->rows-1 : */0;
 	}
 
-	if (!res)
-		notifyUpdate(AdvanceCodeSelectNewRow);
+	if (!res) notifyUpdate(AdvanceCodeSelectNewRow);
+	updateStatus();
 }
 
 void PatternEditorControl::eventKeyDownBinding_NEXT()
@@ -998,8 +1016,8 @@ void PatternEditorControl::eventKeyDownBinding_NEXT()
 		cursor.row = /*wrapAround ? 0 : */pattern->rows-1;
 	}
 
-	if (!res)
-		notifyUpdate(AdvanceCodeSelectNewRow);
+	if (!res) notifyUpdate(AdvanceCodeSelectNewRow);
+
 }
 
 void PatternEditorControl::eventKeyDownBinding_HOME()
@@ -1461,6 +1479,7 @@ void PatternEditorControl::eventKeyDownBinding_CopyTrack()
 	patternEditor->getSelection().start = ss;
 	patternEditor->getSelection().end = se;
 	cursor = cc;
+
 }
 
 void PatternEditorControl::eventKeyDownBinding_PasteTrack()
@@ -1612,6 +1631,7 @@ void PatternEditorControl::eventKeyCharBinding_Copy()
 	cursorCopy = patternEditor->getCursor();
 
 	patternEditor->copy(PatternEditor::ClipBoardTypeSelection);
+	patternEditor->resetSelection(); // prevent user from accidentally [del]eting last selection while in another column
 }
 
 void PatternEditorControl::eventKeyCharBinding_Paste()
@@ -1625,6 +1645,46 @@ void PatternEditorControl::eventKeyCharBinding_Paste()
 		notifyUpdate();
 }
 
+void PatternEditorControl::eventKeyCharBinding_PasteStep()
+{
+	PatternEditorControl::eventKeyCharBinding_Paste();
+	notifyUpdate(AdvanceCodeColumn); 
+}
+
+void PatternEditorControl::eventKeyCharBinding_PasteStepFill()
+{
+	pp_int32  inner        = 0;
+	// remember cursor position
+	cursorCopy = patternEditor->getCursor();
+	pp_int32 row_current   = cursorCopy.row;
+	pp_uint8 times         = 0;
+	PatternEditor::Selection selection = patternEditor->getSelection();
+
+	if( !hasValidSelection() || selection.end.row - selection.start.row < 1 ){ 
+		return;
+	} 
+	pp_int32 row_selection = selection.start.row; 
+	pp_uint32 nSelectedRows = selection.end.row - selection.start.row;
+
+	for( pp_uint8 i = row_selection; i < patternEditor->getNumRows(); i += nSelectedRows ){
+		times++;
+	}
+
+	cursorCopy.row = row_selection;
+	patternEditor->setCursor(cursorCopy);
+	eventKeyCharBinding_Copy();
+
+	for( pp_uint8 i = 0; i < times; i++ ){ // repeat till end of pattern
+		PatternEditorControl::eventKeyCharBinding_PasteStep();
+		cursorCopy = patternEditor->getCursor();
+		cursorCopy.row += nSelectedRows+1; 
+		patternEditor->setCursor(cursorCopy);
+	}
+	cursorCopy.row = row_current;
+	patternEditor->setCursor(cursorCopy);
+	notifyUpdate();
+}
+
 void PatternEditorControl::eventKeyCharBinding_TransparentPaste()
 {
 	if (patternEditor->getCursor() == cursorCopy && menuInvokeChannel != -1)
@@ -1653,6 +1713,11 @@ void PatternEditorControl::eventKeyCharBinding_SelectAll()
 	}
 }
 
+void PatternEditorControl::eventKeyDownBinding_SelectColumn()
+{
+	patternEditor->selectColumn();
+}
+
 void PatternEditorControl::eventKeyCharBinding_MuteChannel()
 {
 	muteChannels[patternEditor->getCursor().channel] = !muteChannels[patternEditor->getCursor().channel];
@@ -1672,3 +1737,123 @@ void PatternEditorControl::eventKeyCharBinding_Interpolate()
 {
 	patternEditor->interpolateValuesInSelection();
 }
+
+void PatternEditorControl::updateStatus()
+{
+	pp_int32 eff  = 0;
+	pp_int32 op   = 0;
+	pp_uint32 op1 = 0;
+	pp_uint32 op2 = 0;
+	char fxchar[2];
+	char label[64];
+	status = "";
+	PatternEditorTools::Position& cursor = patternEditor->getCursor();
+	patternTools.setPosition( patternEditor->getPattern(), cursor.channel, cursor.row);
+
+	switch( cursor.inner ){
+		case 0: patternTools.getNoteName(label, patternTools.getNote()); break;
+		case 1: 
+		case 2: op = patternTools.getInstrument();
+				if( op > 0 ){
+					sprintf(label, op > 0 ? "%i" : "",op); 
+					status = label;
+				}
+				break;  // to vol-cmd
+											   //
+		case 5: {
+					patternTools.getFirstEffect(eff, op); // important: call before getNextEffect
+					patternTools.getNextEffect(eff, op);				
+					patternTools.convertEffectsToFT2(eff, op);
+					if( eff != 0 ){
+						patternTools.getEffectName( fxchar, eff);
+						patternTools.getEffectDescription(label, fxchar[0] );
+						status = PPString(fxchar);
+						status.append(" = ");
+						status.append(label);
+					}
+					break;
+				}
+
+		case 3:
+		case 4:   // fx1
+		case 6: 
+		case 7: { // fx2
+					bool isFX1 = cursor.inner == 3 || cursor.inner == 4;
+					bool isFX2 = !isFX1;
+					char param = cursor.inner == 6 || cursor.inner == 4 ? '1' : '2';
+					patternTools.getFirstEffect(eff, op); // important: call before getNextEffect
+					if( isFX2 ) patternTools.getNextEffect(eff, op);				
+					patternTools.convertEffectsToFT2(eff, op);
+					op1 = patternTools.getNibble( op, PatternEditor::NibbleTypeHigh );
+					op2 = patternTools.getNibble( op, PatternEditor::NibbleTypeLow );
+					patternTools.getEffectName( fxchar, eff);
+					if( eff != 0 ){
+						sprintf(label,"%i",op);
+						status = PPString(label);
+
+						switch( fxchar[0] ){
+							case '0': sprintf(label,"semitone offset%c", param); break;
+							case '1': sprintf(label,"porta up speed");   break;
+							case '2': sprintf(label,"porta down speed"); break;
+							case '3': sprintf(label,"porta note speed"); break;
+							case '4': sprintf(label,"vibrato %s",        param == '1' ? "speed" : "depth" ); break;
+							case '5': sprintf(label,"portafade %s", param == '1' ? "up" : "down" ); break;
+							case '6': sprintf(label,"vibrafade %s", param == '1' ? "up" : "down" ); break;
+							case '7': sprintf(label,"tremolo  %s",       param == '1' ? "speed" : "depth" ); break;
+							case '8': sprintf(label,"pan 0=left ff=right");  break;
+							case '9': sprintf(label,"samplestart 0-FF"); break;
+							case 'A': sprintf(label,"fade speed %s",     isFX1 ? " " : param == '1' ? "up" : "down" ); break;
+							case 'B': sprintf(label,"song position");    break;
+							case 'C': sprintf(label,"note volume");      break;
+							case 'D': sprintf(label,"row next pattern"); break;
+							case 'E': {
+										  switch( op1 ){
+											case 1: sprintf(label, param == '1' ? "fine porta up" : "fporta speed"); break;
+											case 2: sprintf(label, param == '1' ? "fine porta down" : "fporta speed"); break;
+											case 3: sprintf(label, param == '1' ? "glissando" : "not supported"); break;
+											case 4: sprintf(label, param == '1' ? "vibrato control" : "not supported"); break;
+											case 5: sprintf(label, param == '1' ? "note fine-tune" : "fine-tune value"); break;
+											case 6: sprintf(label, param == '1' ? "pattern loop" : "pattern loop start=0 / times"); break;
+											case 7: sprintf(label, param == '1' ? "tremolo control" : "not supported"); break;
+											case 8: sprintf(label, param == '1' ? "note pan pos" : "dont use this"); break;
+											case 9: sprintf(label, param == '1' ? "retrigger note" : "retrigger interval"); break;
+											case 10: sprintf(label, param == '1' ? "fine fade up" : "finefade speed"); break;
+											case 11: sprintf(label, param == '1' ? "fine fade down" : "finefade speed"); break;
+											case 12: sprintf(label, param == '1' ? "note cut" : "note cut tick number"); break;
+											case 13: sprintf(label, param == '1' ? "note delay" : "note cut tick number"); break;
+											case 14: sprintf(label, param == '1' ? "pattern delay" : "pattern delay rows"); break;
+											default: sprintf(label,"not supported"); break;
+										  }
+										  break;
+									  }
+							case 'F': sprintf(label,"spd < 20 > bpm"); break;
+							case 'G': sprintf(label,"global volume"); break;
+							case 'H': sprintf(label,"global fade %s", param == '1' ? "up" : "down"); break;
+							case 'L': sprintf(label,"envelope tick"); break;
+							case 'P': sprintf(label,"pan speed %s", isFX1 ? " " : param == '1' ? "R" : "L"); break;
+							case 'R': sprintf(label,"%s", param == '1' ? "retrig volfade speed" : "retrigger interval"); break;
+							case 'T': sprintf(label,"%s", param == '1' ? "tremor ticks on" : "tremor ticks off"); break;
+							case 'X': {
+										  switch( op1 ){
+											case 1: sprintf(label,"%s", param == '1' ? "xfine porta up" : "speed"); break;
+											case 2: sprintf(label,"%s", param == '1' ? "xfine porta down" : "speed"); break;
+											default: sprintf(label,"not supported"); break;
+										  }
+										  break;
+									  }
+						}
+						status.append(" = ");
+						status.append(label);
+					}
+					break;
+				}
+	}
+}
+
+void PatternEditorControl::roundRobin()
+{
+	if( rr.ins_size > 0 ){
+		rr.ins_index = (rr.ins_index + 1) % (rr.ins_size+1);
+	}
+}
+
