@@ -68,6 +68,7 @@
 #include "DialogHandlers.h"
 #include "DialogChannelSelector.h"
 #include "DialogZap.h"
+#include "DialogSliders.h"
 // Helper class to invoke tools which need parameters
 #include "ToolInvokeHelper.h"
 
@@ -270,8 +271,9 @@ bool Tracker::isEditingCurrentOrderlistPattern()
 
 pp_int32 Tracker::getInstrumentToPlay(pp_int32 note, PlayerController*& playerController)
 {
-	if (PPControl* ctrl = screen->getModalControl())
-	{
+  PPControl* ctrl = screen->getModalControl();
+	if (dialog != NULL && ctrl && screen->hasFocusModal() ) // popupdialogs have no use here (+should allow keyjazz for fxdialogs)
+    {
 		note--;
 	
 		PPContainer* container = static_cast<PPContainer*>(ctrl);
@@ -283,7 +285,7 @@ pp_int32 Tracker::getInstrumentToPlay(pp_int32 note, PlayerController*& playerCo
 		PPListBox* listBoxDstSmp = static_cast<PPListBox*>(container->getControlByID(INSTRUMENT_CHOOSER_LIST_DST2));
 		PPListBox* listBoxDstModule = static_cast<PPListBox*>(container->getControlByID(INSTRUMENT_CHOOSER_LIST_DST3));
 		
-		if (!listBoxSrc || !listBoxDst)
+		if ( screen->getClassic() && (!listBoxSrc || !listBoxDst) )
 			return -1;
 
 		PPListBox* focusedListBox = static_cast<PPListBox*>(container->getFocusedControl());
@@ -300,6 +302,7 @@ pp_int32 Tracker::getInstrumentToPlay(pp_int32 note, PlayerController*& playerCo
 				return focusedListBox->getSelectedIndex() + 1;
 		}
 		
+		if (!listBoxSrc || !listBoxDst)
 		// focus is on instruments		
 		if (focusedListBox == listBoxSrc ||
 			focusedListBox == listBoxSrcModule)
@@ -345,7 +348,8 @@ pp_int32 Tracker::getInstrumentToPlay(pp_int32 note, PlayerController*& playerCo
 			return -1;
 		}
 		
-		return focusedListBox->getSelectedIndex() + 1;
+		return screen->getClassic() ? focusedListBox->getSelectedIndex() + 1
+						            : getPatternEditorControl()->isInstrumentEnabled() ? listBoxInstruments->getSelectedIndex() + 1 : 0;
 	}
 	else
 	{
@@ -568,9 +572,8 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 			caughtMouseInUpperLeftCorner = false;
 	}
 #endif
-	else if (event->getID() == eCommand || event->getID() == eCommandRepeat)
+	else if (event->getID() == eCommand)
 	{
-
 		switch (reinterpret_cast<PPControl*>(sender)->getID())
 		{
 			// test
@@ -646,6 +649,14 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 				break;
 			}
 
+			case BUTTON_PATTERN_SHARPFLAT:
+			{
+				if (event->getID() != eCommand)
+					break;
+				eventKeyDownBinding_ToggleSharpFlat();
+				break;
+			}
+
 			case BUTTON_ABOUT_PROSPECTIVE:
 			{
 				if (event->getID() != eCommand)
@@ -761,13 +772,19 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 				playerLogic->stopSong();
 				break;
 
+			case MAINMENU_HELP:
+				eventKeyDownBinding_InvokeHelp();
+				break;
+
 			// -------- ZAP message box ---------------------
 			case MAINMENU_ZAP:
 				if (event->getID() != eCommand)
 					break;
 
-				if (dialog)
+				if (dialog){
 					delete dialog;
+					dialog = NULL;
+				}
 				
 				if (responder)
 					delete responder;
@@ -1119,6 +1136,19 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 				screen->update();
 				break;
 
+			case BUTTON_PATTERN_CAPTURE:
+				eventKeyDownBinding_InvokePatternCapture();
+				eventKeyDownBinding_InvokeMainScreen();
+				eventKeyDownBinding_InvokeSectionSamples();
+				break;
+
+			case BUTTON_PATTERN_CAPTURE_OVERDUB:
+				eventKeyDownBinding_InvokePatternCaptureOverdub();
+				eventKeyDownBinding_InvokeMainScreen();
+				eventKeyDownBinding_InvokeSectionSamples();
+				break;
+
+
 #ifdef __LOWRES__
 			// go to next order
 			case BUTTON_JAMMENU_NEXTORDERLIST:
@@ -1187,11 +1217,20 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 					getPatternEditorControl()->setCurrentInstrument(listBoxInstruments->getSelectedIndex() + 1);
 					updateSampleEditorAndInstrumentSection(false);
 				}
-				
 				screen->update();
 				break;
 			}
 
+			case BUTTON_INSTRUMENTS_ROUNDROBIN:{
+				PPButton* button = reinterpret_cast<PPButton*>(sender);
+			    PatternEditorControl::RoundRobin *rr = getPatternEditorControl()->getRoundRobin();
+				rr->button = button;
+				updateRoundRobin(true);
+				screen->paintControl(button);
+				screen->update();
+				break;
+		    }
+
 			case BUTTON_TAB_OPEN:
 			{
 				eventKeyDownBinding_OpenTab();
@@ -1278,8 +1317,10 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 			// new instrument has been selected
 			case LISTBOX_INSTRUMENTS:
 			{
+			    PatternEditorControl::RoundRobin *rr = getPatternEditorControl()->getRoundRobin();
 				pp_int32 index = *((pp_int32*)event->getDataPtr()) + 1;
 				selectInstrument(index);		
+				updateRoundRobin(false);
 				screen->update();
 				break;
 			}
@@ -1710,10 +1751,17 @@ pp_int32 Tracker::handleEvent(PPObject* sender, PPEvent* event)
 						return 1;
 						
 					// Not wrapped at all... Just calculate new position within pattern and update player position
-					case PatternEditorControl::AdvanceCodeSelectNewRow:
+					case PatternEditorControl::AdvanceCodeSelectNewRow:{
 						if (shouldFollowSong() && 
 							isEditingCurrentOrderlistPattern())
 							updateSongRow();
+						else backtraceInstrument(0,true);
+						break;
+					}
+
+					case PatternEditorControl::AdvanceCodeColumn:
+						backtraceInstrument(0,false);
+						getPatternEditorControl()->updateStatus();
 						break;
 				}
 				break;
@@ -2128,6 +2176,7 @@ void Tracker::updateSongRow(bool checkFollowSong/* = true*/)
 {
 	if (checkFollowSong && !shouldFollowSong())
 		return;
+	else getPatternEditorControl()->updateStatus();
 		
 	mp_sint32 row = getPatternEditorControl()->getCurrentRow();
 	mp_sint32 pos = listBoxOrderList->getSelectedIndex();
@@ -2144,6 +2193,7 @@ void Tracker::selectInstrument(pp_int32 instrument)
 	getPatternEditorControl()->setCurrentInstrument(instrument);
 	
 	sectionTranspose->setCurrentInstrument(instrument, false);
+	enableInstrument(true); // important: re-enable in case turned off (by numpad 0)
 
 	updateSamplesListBox(false);
 
@@ -2152,9 +2202,10 @@ void Tracker::selectInstrument(pp_int32 instrument)
 	updateSampleEditor(false);
 	// update instrument/sample editor
 	sectionInstruments->update(false);
-
 	for (pp_int32 i = 0; i < sections->size(); i++)
 		sections->get(i)->notifyInstrumentSelect(instrument);
+
+	screen->update();
 }
 
 void Tracker::fillInstrumentListBox(PPListBox* listBox, ModuleEditor* moduleEditor/* = NULL*/)
@@ -2345,6 +2396,7 @@ void Tracker::expandOrderlist(bool b)
 
 void Tracker::flipSpeedSection()
 {
+  if( !screen->getClassic() ) return;
 	PPContainer* container = static_cast<PPContainer*>(screen->getControlByID(CONTAINER_SPEED));
 	ASSERT(container);
 
@@ -2519,6 +2571,7 @@ pp_uint32 Tracker::fileTypeToHint(FileTypes type)
 		case FileTypes::FileTypeSongAllSamples:
 		case FileTypes::FileTypeSampleWAV:
 		case FileTypes::FileTypeSampleIFF:
+		case FileTypes::FileTypeSampleSF2:
 			return DecompressorBase::HintSamples;
 
 		default:
@@ -2727,6 +2780,10 @@ bool Tracker::loadTypeFromFile(FileTypes eType, const PPSystemString& fileName,
 		case FileTypes::FileTypeSongAllInstruments:
 		{
 			loadingParameters.res = moduleEditor->loadInstrument(loadingParameters.filename, listBoxInstruments->getSelectedIndex());
+			if( loadingParameters.filename.compareExtensions( PPString(".sf2") ) == 0 ){ 
+				sectionSamples->getSampleEditorControl()->invokeSoundfont();
+				loadingParameters.res = true;
+			}
 			sectionInstruments->updateAfterLoad();
 			break;
 		}
@@ -2744,8 +2801,10 @@ bool Tracker::loadTypeFromFile(FileTypes eType, const PPSystemString& fileName,
 			else if (numSampleChannels > 1 && 
 					 !settingsDatabase->restore("AUTOMIXDOWNSAMPLES")->getIntValue())
 			{
-				if (dialog)
+				if (dialog){
 					delete dialog;
+					dialog = NULL;
+				}
 				
 				if (responder)
 					delete responder;
@@ -2783,6 +2842,9 @@ bool Tracker::loadTypeFromFile(FileTypes eType, const PPSystemString& fileName,
 																 listBoxInstruments->getSelectedIndex(), 
 																 listBoxSamples->getSelectedIndex(), 
 																 chnIndex);
+			if( loadingParameters.filename.compareExtensions( PPString(".sf2") ) == 0 ){
+				sectionSamples->getSampleEditorControl()->invokeSoundfont();
+			}
 
 			sectionSamples->updateAfterLoad();
 			break;
@@ -3080,16 +3142,11 @@ void Tracker::saveType(FileTypes eType)
 	{
 		prepareLoadSaveUI();
 	
-		sectionDiskMenu->selectSaveType(eType);
-		
-		if (eType == FileTypes::FileTypeSongWAV)
-			sectionDiskMenu->setModuleTypeAdjust(false);
-		
 		eventKeyDownBinding_InvokeSectionDiskMenu();
-		
-		if (eType == FileTypes::FileTypeSongWAV)
-			sectionDiskMenu->setModuleTypeAdjust(true);
-	
+
+		sectionDiskMenu->setModuleTypeAdjust(true);
+		sectionDiskMenu->switchState( SectionDiskMenu::BrowseModules );
+
 		finishLoadSaveUI();
 	}
 	else
@@ -3190,8 +3247,10 @@ void Tracker::buildMODSaveErrorWarning(pp_int32 error)
 									 "* Volume column is not used        \n"\
 									 "* Only effects between 0 and F     \n\nSave anyway?"};
 	
-	if (dialog)
+	if (dialog){
 		delete dialog;
+		dialog = NULL;
+	}
 	
 	if (responder)
 		delete responder;
@@ -3234,3 +3293,51 @@ void Tracker::signalWaitState(bool b)
 	screen->signalWaitState(b, TrackerConfig::colorThemeMain);	
 }
 
+
+void Tracker::backtraceInstrument(pp_uint8 channelIncrement, bool currentPosOnly ){
+	PatternEditor *p = getPatternEditor();
+	if( !p->getInstrumentBackTrace() ) return;
+	// when switching channels: visually auto-switch instrument
+	// (=less misstakes during liveperformance / ASCIISTEP16 stepsequencer mode)                
+	PatternTools patternTools;
+	PatternEditorTools::Position cursor = p->getCursor();
+	pp_int8 chan = cursor.channel + channelIncrement;
+	pp_int32 ins  = -1;
+
+    // check current position	
+	patternTools.setPosition( p->getPattern(), chan, cursor.row);
+	if ( patternTools.getNote() != 0 ){ ins = patternTools.getInstrument();	}
+
+	// backtrace last instrument on pattern-channel
+	if( ins < 1 && !currentPosOnly ){
+		for (pp_int32 i = cursor.row; i >= 0; i--)
+		{
+			patternTools.setPosition( p->getPattern(), chan, i);
+			pp_int32 lastIns = patternTools.getInstrument();	
+			if ( patternTools.getNote() != 0 && lastIns > 0){
+				ins = lastIns;
+				break;
+			}
+		}
+	}
+
+	// find future note if backtrace wasn't possible
+	if( ins < 1 && !currentPosOnly ){
+		for (pp_int32 i = cursor.row; i < p->getNumRows(); i++)
+		{
+			patternTools.setPosition( p->getPattern(), chan, i);
+			pp_int32 lastIns = patternTools.getInstrument();	
+			if ( patternTools.getNote() != 0 && lastIns > 0){
+				ins = lastIns;
+				break;
+			}
+		}
+	}
+
+	if( ins < 1 ) return;
+	getPatternEditor()->setCurrentInstrument(ins);
+	moduleEditor->setCurrentInstrumentIndex(ins-1);
+	listBoxInstruments->setSelectedIndex(ins-1);
+	updateInstrumentsListBox(true);
+	updateSampleEditor(true);
+}
