@@ -45,6 +45,7 @@
 #include "SectionSamples.h"
 #include "SectionHDRecorder.h"
 #include "SectionQuickOptions.h"
+#include "SectionSettings.h"
 #include "TabHeaderControl.h"
 #include "PPOpenPanel.h"
 #include "TitlePageManager.h"
@@ -53,15 +54,30 @@
 
 void Tracker::updateAboutToggleButton(pp_int32 id, bool b, bool repaint/* = true*/)
 {
-	PPContainer* container = static_cast<PPContainer*>(screen->getControlByID(CONTAINER_ABOUT));
-	ASSERT(container);
-	PPButton* button = static_cast<PPButton*>(container->getControlByID(id));
-	if (button == NULL)
-		return;
-	button->setPressed(b);	
-	
-	if (repaint)
-		screen->paintControl(button);
+  pp_int32 btns[2] = { CONTAINER_ABOUT, CONTAINER_MENU };
+  for( int i = 0; i < 2; i++ ){
+    PPContainer* container = static_cast<PPContainer*>(screen->getControlByID(btns[i]));
+    ASSERT(container);
+    PPButton* button = static_cast<PPButton*>(container->getControlByID(id));
+    if (button == NULL)
+      return;
+    button->setPressed(b);	
+    
+    if (repaint)
+      screen->paintControl(button);
+  }
+}
+
+void Tracker::updatePlayButtons(pp_int32 id, bool b, bool repaint/* = true*/)
+{
+  pp_int32 btns[3] = { MAINMENU_PLAY_SONG, MAINMENU_PLAY_PATTERN, MAINMENU_PLAY_POSITION };
+  for( int i = 0; i < 3; i++ ){
+    PPButton *btn = static_cast<PPButton*>(screen->getControlByID(btns[i]));
+    btn->setPressed( id == btns[i] && b);	
+    
+    if (repaint)
+      screen->paintControl(btn);
+  }
 }
 
 bool Tracker::updatePianoControl(PianoControl* pianoControl)
@@ -474,6 +490,8 @@ void Tracker::updateInstrumentsListBox(bool repaint)
 	listBoxInstruments->saveState();
 				
 	listBoxInstruments->clear();
+
+	listBoxInstruments->setHexIndex( true ); // always enfore, cauz patterneditor only supports hex instruments
 	
 	fillInstrumentListBox(listBoxInstruments);
 				
@@ -794,6 +812,7 @@ void Tracker::updateRecordButton(PPContainer* container, const PPColor& pColor)
 		if (button->isVisible() && (&pColor) != button->getTextColor())
 		{
 			button->setTextColor(pColor);
+			button->setPressed( !button->isPressed() );
 			screen->paintControl(button);
 		}
 	}
@@ -1061,6 +1080,14 @@ void Tracker::updateAfterLoad(bool loadResult, bool wasPlaying, bool wasPlayingP
 	}
 	
 	updateSongInfo(false);
+
+	if( settingsDatabase->restore("LIMITRESET")->getBoolValue() ){ // reset limiter
+		TMixerSettings newMixerSettings;
+		settingsDatabase->store("LIMITDRIVE",0);
+		sectionSettings->saveCurrentMixerSettings(newMixerSettings);
+		bool res = playerMaster->applyNewMixerSettings(newMixerSettings, true);
+		sectionSettings->update(true);
+	}
 }
 
 void Tracker::updateAfterTabSwitch()
@@ -1133,3 +1160,60 @@ void Tracker::updateAfterTabSwitch()
 	screen->paint(true);
 }
 
+void Tracker::updateRoundRobin( bool buttonPress )
+{
+	PatternEditorControl::RoundRobin *rr = getPatternEditorControl()->getRoundRobin();
+	bool reset   = false;
+	bool enabled = rr->ins_size > 0;
+	switch( rr->userOnboarded ){
+		case 0: {
+					if( buttonPress ){
+						showMessageBox(MESSAGEBOX_UNIVERSAL, "roundrobin: select other instrument", MessageBox_OK);
+						rr->userOnboarded++;
+					}
+				}
+				break;
+
+		case 1: PPString msg = PPString("roundrobin: now enabled");
+				showMessageBox(MESSAGEBOX_UNIVERSAL, msg, MessageBox_OK);
+				rr->userOnboarded++;
+				break;
+	}
+	if( rr->userSelecting ){
+		if( !buttonPress ){
+			// update RR values
+			rr->ins_end  = listBoxInstruments->getSelectedIndex() + 1;
+			if( rr->ins_end < rr->ins_start ){
+				int tmp = rr->ins_start;
+				rr->ins_start = rr->ins_end;
+				rr->ins_end  = tmp;
+			}
+			rr->ins_size  = rr->ins_end - rr->ins_start;
+			if( rr->button ){
+				rr->button->setTextColor( TrackerConfig::colorSampleEditorWaveform );
+				rr->button->setPressed(true);
+			}
+			rr->userSelecting = false;
+			listBoxInstruments->highlightRange( rr->ins_start-1, rr->ins_end-1 );
+		}else reset = true;
+	}else{
+		if( rr->ins_start == 0 && buttonPress ){
+			rr->ins_start = listBoxInstruments->getSelectedIndex() + 1;
+			if( rr->button ){
+				rr->button->setTextColor( PPUIConfig::getInstance()->getColor(PPUIConfig::ColorDefaultButtonText) );
+				rr->button->setPressed(true);
+			}
+			rr->userSelecting = true;
+		}else reset = true;
+	}
+	if( reset ){
+		rr->ins_size = rr->ins_start = rr->ins_end = 0;
+		rr->userSelecting = false;
+		if( rr->button ){
+			rr->button->setTextColor( PPUIConfig::getInstance()->getColor(PPUIConfig::ColorDefaultButtonText) );
+			rr->button->setPressed(false);
+		}
+		listBoxInstruments->highlightRange( -1, -1 );
+	}
+	if( rr->button ) screen->paint(rr->button);
+}
